local cp = console_print

local bstr = function(id, value)
	return value
end

local ACAM = {}
local update_class, babel, ref_key
local babel_func = function()
	local valid_ver = lib.get_latest("babel", "0", "1.99")
	if valid_ver == "?" then
		return
	end
	
	babel = lib.get_class("babel", valid_ver)
	ref_key = babel.register("plugins/ACAM/lang/", {'en', 'es', 'fr', 'pt'})
	
	bstr = function(id, val)
		return babel.fetch(ref_key, id, val)
	end
	
	ACAM.add_translation = function(path, lang_code)
		babel.add_new_lang(ref_key, path, lang_code)
	end
	
	update_class()
end

local gkrs = gkini.ReadString
local gkws = gkini.WriteString

local is_running = false
local config = {
	active = gkrs("ACAM", "active", "NO"), --is the timed switcher running or not
	tick_time = tonumber(gkrs("ACAM", "tick_time", "1")) or 1,
		--time between ACAM updates; increase for more delay if chuggingin
	
	time_style = gkrs("ACAM", "time_style", "basic"),
		--[[
		affects auto_timing calculation
		
		manual: do not calculate automatically
		basic: assume all ports have same; use first timing found
		average: average timing between all equipment
		slowest: use slowest timing between all equipment
		]]--
	time_subgroup = gkrs("ACAM", "time_subgroup", "x2"),
		--[[
			used for obtaining preset timings
			x2: user has 2 group setup
			x3: 3 group setup (valk and maud)
			x4: 4 group setup (hornet and ragnarok)
		]]--
	timing = gkrs("ACAM", "timing", "001"), --in ms
		--the time between group changes
	group_count = tonumber(gkrs("ACAM", "group_count", "4")) or 4,
		--how many weapon groups to iterate
		
	timings_preset_file = gkrs("ACAM", "timings_preset_file", "plugins/ACAM/timings.ini"),
		--allow providing custom defaults
	
	ignore_args = "NO"
		--disable CLI, only set from public class functions
}

update_class = function()
	local public = {
		CCD1 = true,
		smart_config = {
			title = "ACAM" .. bstr(2, "Settings"),
			--Advanced Chain Automation Module
			cb = function(id, val)
				if config[id] then
					config[id] = val
					gkws("ACAM", id, val)
				end
			end,
			acam_descriptor1 = {
				type = "text",
				display = bstr(1, "Advanced Chainfire Automation Module"),
			},
			spacer = {
				type = "spacer",
			},
			active = {
				type = "toggle",
				display = bstr(3, "Activate ACAM process while undocked"),
				[1] = config.active,
			},
			time_style = {
				type = "dropdown",
				display = bstr(4, "Automatically calculate timing"),
				default = (config.time_style == bstr(5, "manual") and 1) or (config.time_style == bstr(6, "basic") and 2) or (config.time_style == bstr(7, "average") and 3) or (config.time_style == bstr(8, "slowest") and 4) or 1,
				bstr(5, "manual"),
				bstr(6, "basic"),
				bstr(7, "average"),
				bstr(8, "slowest"),
			},
			time_subgroup = {
				type = "dropdown",
				display = "subgroup for calculation",
				default = (config.time_subgroup == "x2" and 1) or (config.time_subgroup == "x3" and 2) or (config.time_subgroup == "x4" and 3) or 1,
				"x2",
				"x3",
				"x4",
			},
			group_count = {
				type = "dropdown",
				display = bstr(9, "Number of chainfired weapon groups"),
				default = (config.group_count == "1" and 1) or (config.group_count == "2" and 2) or (config.group_count == "3" and 3) or (config.group_count == "4" and 4) or (config.group_count == "5" and 5) or 6,
				"1",
				"2",
				"3",
				"4",
				"5",
				"6",
			},
			"acam_descriptor1",
			"spacer",
			"active",
			"spacer",
			"time_style",
			"time_subgroup",
			"spacer",
			"group_count",
		},
		commands = {
			"/ACAM",
		},
		manifest = {
			"plugins/ACAM/acam.lua",
		},
		description = bstr(10, "ACAM provides a drop-in chainfire solution for all players"),
	}
	
	for k, v in pairs(public) do
		ACAM[k] = v
	end
	
	lib.set_class("ACAM", "0", ACAM)
end

local gkgms = gkmisc.GetGameTime

local acam_tick = Timer()
local tick_accumulator = 0 --time until next group switch (increasing)
local last_tick = gkgms() --millisecond time of last call
local firing_group = 1 --index of current weapon group
local acam_process --self reference
acam_process = function()
	local tick_reset = tonumber(config.tick_time) or 1
	
	acam_tick:SetTimeout(tick_reset, acam_process)
	
	if (config.active == "NO") or (PlayerInStation()) or (not IsConnected()) or (not GetActiveShipName()) then
		--chainfire disabled temp or player is not in a firing situation
		return
	end
	
	tick_accumulator = tick_accumulator + (gkgms() - last_tick)
	last_tick = gkgms()
	
	if tick_accumulator < (tonumber(config.timing) or 1000) then
		--not time to switch group
		return
	end
	--time to switch groups
	
	tick_accumulator = 0
	firing_group = firing_group + 1
	if firing_group > (tonumber(config.group_count) or 2) then
		firing_group = 1
	end
	
	gkinterface.GKProcessCommand("Weapon" .. tostring(firing_group))
	
end







local exchange_spaces = function(instr)
	return string.gsub(tostring(instr), " ", "_")
end

--public functions (if LME is available); also used by lite interface
ACAM.get_timing = function(weapon_string, itemID)
	weapon_string = exchange_spaces(weapon_string)
	local weapon_timing = gkrs("ACAM" .. config.time_subgroup, weapon_string, "?")
	local timing_type = "custom"
	
	if weapon_timing == "?" then
		--no custom weapon timing, get preset from timings.ini
		weapon_timing = gkini.ReadString2("ACAM" .. config.time_subgroup, weapon_string, "?", config.timings_preset_file)
		timing_type = "preset"
	end
	
	if weapon_timing == "?" and itemID then
		--no preset known for this weapon, try weapon ID if it exists
		if GetAddonItemCooldownInfo(itemID) ~= 0 then
			weapon_timing = tostring((GetAddonItemCooldownInfo(itemID) or 1000)/ (tonumber(config.group_count) or 2))
		end
		timing_type = "guess"
	end
	
	if weapon_timing == "?" then
		--no preset known for this weapon, assume 500 and pass on as error
		weapon_timing = "200"
		timing_type = "none"
	end
	
	return weapon_timing, timing_type
end

ACAM.save_timing = function(weapon_string, time_val, subgroup)
	weapon_string = exchange_spaces(weapon_string)
	time_val = tostring(time_val)
	subgroup = tostring(subgroup)
	
	gkws("ACAM" .. subgroup, weapon_string, time_val)
end

ACAM.load_timing = function(weapon_string, time_val, subgroup)
	weapon_string = exchange_spaces(weapon_string)
	time_val = tostring(time_val)
	subgroup = tostring(subgroup)
	
	return gkrs("ACAM" .. subgroup, weapon_string, time_val)
end

ACAM.set_config = function(cfg, val)
	cfg = tostring(cfg)
	val = tostring(val)
	
	if config[cfg] then
		config[cfg] = val
		gkws("ACAM", cfg, val)
	end
end

ACAM.get_config = function(cfg)
	return config[cfg]
end

ACAM.calculate = function()
	--using the current settings and current ship, calculate the timing
	
	cp("ACAM time calculation method is: " .. config.time_style)
	
	if config.time_style == bstr(5, "manual") then
		--manual mode is being used, don't calculate
		return tonumber(config.timing) or 20
	end
	
	local test_ship_exist = GetActiveShipName()
	if not test_ship_exist then
		return false
	end
	
	local num_ports = GetActiveShipNumAddonPorts() - 2
		--[[
			port 1: powercell
			one port unaccounted for, leftover from engine?
			port 2: weapon
			...
			on an EC-107, there are "4" ports
		]]--
	local port_setup = {}
	for i=2, num_ports + 1 do
		local item_id = GetActiveShipItemIDAtPort(i)
		local item_name = GetInventoryItemName(item_id)
		table.insert(port_setup, {item_name, item_id} or {"???", 0})
	end
	
	local start_timing = -1
	
	for port_index, port_obj in ipairs(port_setup) do
		port_obj[1] = port_obj[1] or "?"
		local weapon_timing, timing_type = ACAM.get_timing(port_obj[1] or "?", port_obj[2] or 0)
		weapon_timing = tonumber(weapon_timing) or 20
		
		if timing_type == "none" then
			print(port_obj[1] .. " " .. bstr(11, "not recongised by ACAM"))
		elseif timing_type == "guess" then
			print("Addon " .. port_obj[1] .. " " .. bstr(11, "not recongised by ACAM") .. "; " .. bstr(12, "attempted to guess timing via weapon delay") .. " >> " .. tostring(weapon_timing))
		end
		
		if config.time_style == bstr(6, "basic") then
			--basic: First weapon timing is used for all
			start_timing = weapon_timing
			break
		elseif config.time_style == bstr(7, "average") then
			--average: all weapon timings are averaged
			if start_timing < 0 then
				start_timing = weapon_timing
			end
			start_timing = (start_timing + weapon_timing) / 2
		elseif config.time_style == bstr(8, "slowest") then
			--slowest: slowest weapon timing is used for all
			if weapon_timing > start_timing then
				start_timing = weapon_timing
			end
		else
			cp("unrecognised config.time_style, assuming basic method")
			start_timing = weapon_timing
			break
		end
	end
	
	return start_timing
	
end

ACAM.setup_weapons = function(callback)
	local equipment = {}
		--this table holds group trigger stuff
	local num_ports = GetActiveShipNumAddonPorts() - 2
		--get all equipment on ship
	for i=2, num_ports + 1 do
		local group_config = {
			[i] = true,
		} --one weapon per group. port_id is i.
		
		table.insert(equipment, group_config)
			--key of table is the group to be bound
			--value of table is the group bindings
	end
	
	if not callback then
		callback = function() end
	end
	
	ConfigureMultipleWeaponGroups(equipment)
	callback()
		--I thought configureMultiple... was supposed to allow a callback argument, but it never got called. /shrug/ ?
end

local auto_set_timing = function()
	--ensure that the number of groups doesn't exceed 6
	if (tonumber(config.group_count) or 10) > 6 then
		config.group_count = "2"
	end
	
	--calculate timing if not set to manual
	local new_timing = ACAM.calculate()
	if new_timing then
		ACAM.set_config("timing", tostring(new_timing))
	end
	
	--init the process tick
	acam_process()
	is_running = true
end

RegisterEvent(auto_set_timing, "LEAVING_STATION")



--interface lite
local cl_handler = function(args)
	--[[
		?/help
		save [name] [nil/subgroup]
		load [name] [nil/subgroup]
		toggle [nil/ON/OFF]
		set
			auto [calc option]
			timing [value]
			subgroup [2/3/4]
			gcount [1-6]
	]]--
	if args[1] == "?" or args[1] == "help" then
		print("ACAM, or 'Advanced Chainfire Automation Module', provides a drag-and-drop chainfire solution suitable for all pilots")
		print("Commands you can use with /acam")
		print("	<no arguments>: open the lite interface")
		print("	? or help: show this help message")
		print("	save [name] [subgroup]")
		print("		Save the current timing to the config file. subgroup is optional")
		print("	load [name] [subgroup]")
		print("		Load a custom timing in the config file. subgroup is optional")
		print("	toggle [on or off]")
		print("		turn on or off ACAM. State is optional")
		print("	setup")
		print("		adjust weapon triggers for chainfire")
		print("	set [option] [value]")
		print("		auto [manual or basic or average or slowest]")
		print("			sets the automatic timing calculation method")
		print("			manual: do not auto-calculate timing on undock")
		print("			basic: use timing found for first weapon")
		print("				This is the default")
		print("			average: all known weapon timings are averaged together")
		print("			slowest: the slowest weapon decides timing")
		print("		timing [timing delay]")
		print("			sets a custom chainfire timing")
		print("		subgroup [2 or 3 or 4]")
		print("			sets the active subgroup for automatic timing calculation")
		print("		gcount [1 through 6]")
		print("			sets the number of groups to iterate while chainfiring")
	elseif args[1] == "setup" then
		ACAM.setup_weapons(function()
			print("Weapons have been adjusted across your group ports")
		end)
	elseif args[1] == "save" then
		ACAM.save_timing(args[2], config.timing, args[3] or "")
		print("ACAM: saved timing as " .. exchange_spaces(args[2]))
	elseif args[1] == "load" then
		config.timing = ACAM.load_timing(args[2], config.timing, args[3] or "")
		print("ACAM: loaded timing: " .. config.timing)
	elseif args[1] == "toggle" then
		local state = args[2]
		if not state then
			state = ACAM.get_config("active") == "YES" and "ON" or "OFF"
		end
		ACAM.set_config("active", state == "ON" and "YES" or "NO")
		print("ACAM: active state set to " .. state)
	elseif args[1] == "set" then
		if args[2] == "auto" then
			local err_flag = true
			for _, option in ipairs {
				bstr(5, "manual"),
				bstr(6, "basic"),
				bstr(7, "average"),
				bstr(8, "slowest"),
			} do
				if args[3] == option then
					err_flag = false
					break
				end
			end
			
			if err_flag then
				args[3] = "basic"
			end
			
			ACAM.set_config("time_style", args[3])
			
			print("ACAM: auto-calculation method set to " .. args[3])
		elseif args[2] == "timing" then
			ACAM.set_config("timing", tostring(tonumber(args[3] or 1) or 1) or "1")
			
			print("ACAM: set timing to " .. config.timing)
		elseif args[2] == "subgroup" then
			local err_flag = true
			for _, option in ipairs {
				"2",
				"3",
				"4",
			} do
				if args[3] == option then
					err_flag = false
					break
				end
			end
			
			if err_flag then
				args[3] = "2"
			end
			
			ACAM.set_config("time_subgroup", "x" .. args[3])
			
			print("ACAM: subgroup set to " .. args[3])
		elseif args[2] == "gcount" then
			local err_flag = true
			for _, option in ipairs {
				"1",
				"2",
				"3",
				"4",
				"5",
				"6",
			} do
				if args[3] == option then
					err_flag = false
					break
				end
			end
			
			if err_flag then
				args[3] = "2"
			end
			
			ACAM.set_config("group_count", args[3])
			
			print("ACAM: group count set to " .. args[3])
		else
			print("ACAM: unknown config option " .. args[2])
		end
	else
		print("ACAM: unknown command " .. args[1])
	end
end

ACAM.open = function(_, args)
	
	if args and config.ignore_args ~= "YES" then
		cl_handler(args)
		return
	end
	
	if not is_running then
		auto_set_timing()
	end
	
	local group_time_edit = iup.text {
		value = ACAM.get_config("timing"),
		size = "200x",
		action = function(self)
			ACAM.set_config("timing", self.value)
		end,
	}
	
	local num_group_edit = iup.text {
		value = ACAM.get_config("group_count"),
		size = "200x",
		action = function(self)
			ACAM.set_config("group_count", self.value)
		end,
	}
	
	local active_toggle = iup.stationtoggle {
		title = "",
		value = ACAM.get_config("active") == "YES" and "ON" or "OFF",
		action = function(self)
			ACAM.set_config("active", self.value == "ON" and "YES" or "NO")
		end,
	}
	
	local time_value = ACAM.get_config("time_style")
	local auto_calc_dropdown = iup.stationsublist {
		dropdown = "YES",
		action = function(self, t, i, cv)
			if cv == 1 then
				ACAM.set_config("time_style", t)
			end
		end,
		value = (time_value == bstr(5, "manual") and 1) or (time_value == bstr(6, "basic") and 2) or (time_value == bstr(7, "average") and 3) or (time_value == "slowest" and 4) or 1,
		bstr(5, "manual"),
		bstr(6, "basic"),
		bstr(7, "average"),
		bstr(8, "slowest"),
	}
	
	local time_sub = ACAM.get_config("time_subgroup")
	local subselect_dropdown = iup.stationsublist {
		dropdown = "YES",
		action = function(self, t, i, cv)
			if cv == 1 then
				ACAM.set_config("time_subgroup", t)
			end
		end,
		value = (time_sub == "x2" and 1) or (time_sub == "x3" and 2) or (time_sub == "x4" and 3) or 1,
		"x2",
		"x3",
		"x4",
	}
	
	
	local diag
	diag = iup.dialog {
		topmost = "YES",
		fullscreen = "YES",
		bgcolor = "0 0 0 150 *",
		update_content = function(self)
			group_time_edit.value = ACAM.get_config('timing')
			num_group_edit.value = ACAM.get_config("group_count")
			active_toggle.value = ACAM.get_config("active") == "YES" and "ON" or "OFF"
			
			time_value = ACAM.get_config("time_style")
			auto_calc_dropdown.value = (time_value == bstr(5, "manual") and 1) or (time_value == bstr(6, "basic") and 2) or (time_value == bstr(7, "average") and 3) or (time_value == bstr(8, "slowest") and 4) or 1
			
			time_sub = ACAM.get_config("time_subgroup")
			subselect_dropdown.value = (time_sub == "x2" and 1) or (time_sub == "x3" and 2) or (time_sub == "x4" and 3) or 1
		end,
		iup.hbox {
			iup.fill { },
			iup.vbox {
				iup.fill { },
				iup.hbox {
					iup.label {
						title = "ACAM " .. bstr(13, "settings lite"),
					},
					iup.fill { },
					iup.stationbutton {
						title = bstr(14, "Close"),
						size = "x" .. tostring(Font.Default),
						action = function(self)
							HideDialog(iup.GetDialog(self))
						end,
					},
				},
				iup.stationdoubleframe {
					iup.hbox {
						iup.fill {
							size = "%1",
						},
						iup.vbox {
							iup.fill {
								size = "%3",
							},
							iup.hbox {
								iup.label {
									title = bstr(15, "Run ACAM while undocked") .. ":",
								},
								iup.fill { },
								active_toggle,
							},
							iup.fill {
								size = "%1",
							},
							iup.hbox {
								iup.label {
									title = bstr(16, "Enable automatic timing calculation") .. ":",
								},
								iup.fill { },
								auto_calc_dropdown,
							},
							iup.fill {
								size = "%1",
							},
							iup.hbox {
								iup.label {
									title = bstr(17, "Subgroup for auto-calculation") .. ":",
								},
								iup.fill { },
								subselect_dropdown,
							},
							iup.fill {
								size = "%1",
							},
							iup.hbox {
								iup.label {
									title = bstr(18, "Number of weapon groups to use") .. ":",
								},
								iup.fill { },
								num_group_edit,
							},
							iup.fill {
								size = "%1",
							},
							iup.hbox {
								iup.label {
									title = "ACAM chainfire " .. bstr(19, "timing") .. ":",
								},
								iup.fill { },
								group_time_edit,
							},
							iup.fill {
								size = "%3",
							},
							iup.hbox {
								iup.stationbutton {
									title = bstr(20, "Setup weapons for chainfire"),
									action = function(self)
										local notify_popup = function()
											local subdiag = iup.dialog {
												topmost = "YES",
												fullscreen = "YES",
												bgcolor = "0 0 0 150 *",
												iup.vbox {
													iup.fill { },
													iup.hbox {
														iup.fill { },
														iup.stationdoubleframe {
															iup.hbox {
																iup.fill {
																	size = "%1",
																},
																iup.vbox {
																	iup.fill {
																		size = "%3",
																	},
																	iup.label {
																		title = bstr(21, "Groups have been adjusted"),
																	},
																	iup.fill {
																		size = "%1",
																	},
																	iup.stationbutton {
																		title = "OK",
																		action = function(self)
																			HideDialog(iup.GetDialog(self))
																		end,
																	},
																	iup.fill {
																		size = "%3",
																	},
																},
																iup.fill {
																	size = "%1",
																},
															},
														},
														iup.fill { },
													},
													iup.fill { },
												},
											}
											subdiag:map()
											ShowDialog(subdiag)
										end
										
										ACAM.setup_weapons(notify_popup)
									end,
								},
								iup.fill { },
								iup.stationbutton {
									title = bstr(22, "Save timing"),
									action = function(self)
										--save as (default name: weapon in port 1)
										--save under subgroup of x2, x3, x4
										local itemID = GetActiveShipItemIDAtPort(2)
											--get first weapon's itemID, use as default name
										
										local name_to_save = GetInventoryItemName(itemID)
										local time_to_save = config.timing
										local subg_to_save = config.time_subgroup
										
										local subdiag = iup.dialog {
											topmost = "YES",
											fullscreen = "YES",
											bgcolor = "0 0 0 150 *",
											iup.vbox {
												iup.fill { },
												iup.hbox {
													iup.fill { },
													iup.stationdoubleframe {
														iup.hbox {
															iup.fill {
																size = "%1",
															},
															iup.vbox {
																iup.fill {
																	size = "%3",
																},
																iup.label {
																	title = bstr(24, "Pick a name and subgroup to save this timing"),
																},
																iup.fill {
																	size = "%1",
																},
																iup.hbox {
																	iup.label {
																		title = bstr(25, "name") .. ":",
																	},
																	iup.text {
																		size = "%20x",
																		value = name_to_save,
																		action = function(self)
																			name_to_save = self.value
																		end,
																	},
																},
																iup.hbox {
																	iup.label {
																		title = bstr(26, "subgroup") .. ":",
																	},
																	iup.stationsublist {
																		dropdown = "YES",
																		action = function(self, t, i, cv)
																			if cv == 1 then
																				subg_to_save = t
																			end
																		end,
																		value = (subg_to_save == "x2" and 1) or (subg_to_save == "x3" and 2) or (subg_to_save == "x4" and 3) or 1,
																		"x2",
																		"x3",
																		"x4",
																	},
																},
																iup.fill {
																	size = "%1",
																},
																iup.hbox {
																	iup.fill { },
																	iup.stationbutton {
																		title = bstr(22, "Save"),
																		action = function(self)
																			ACAM.save_timing(name_to_save, time_to_save, subg_to_save)
																			HideDialog(iup.GetDialog(self))
																		end,
																	},
																	iup.stationbutton {
																		title = bstr(27, "Cancel"),
																		action = function(self)
																			HideDialog(iup.GetDialog(self))
																		end,
																	},
																},
																iup.fill {
																	size = "%3",
																},
															},
															iup.fill {
																size = "%1",
															},
														},
													},
													iup.fill { },
												},
												iup.fill { },
											},
										}
										subdiag:map()
										ShowDialog(subdiag)
									end,
								},
								iup.stationbutton {
									title = bstr(23, "Load timing"),
									action = function(self)
										--load from name
										--load from subgroup
										local itemID = GetActiveShipItemIDAtPort(2)
											--get first weapon's itemID, use as default name
										
										local name_to_save = GetInventoryItemName(itemID)
										local time_to_save = config.timing
										local subg_to_save = config.time_subgroup
										
										
										local subdiag = iup.dialog {
											topmost = "YES",
											fullscreen = "YES",
											bgcolor = "0 0 0 150 *",
											iup.vbox {
												iup.fill { },
												iup.hbox {
													iup.fill { },
													iup.stationdoubleframe {
														iup.hbox {
															iup.fill {
																size = "%1",
															},
															iup.vbox {
																iup.fill {
																	size = "%3",
																},
																iup.label {
																	title = bstr(28, "Pick a name and subgroup to load from config"),
																},
																iup.fill {
																	size = "%1",
																},
																iup.hbox {
																	iup.label {
																		title = bstr(25, "name") .. ":",
																	},
																	iup.text {
																		size = "%20x",
																		value = name_to_save,
																		action = function(self)
																			name_to_save = self.value
																		end,
																	},
																},
																iup.hbox {
																	iup.label {
																		title = bstr(26, "subgroup") .. ":",
																	},
																	iup.stationsublist {
																		dropdown = "YES",
																		action = function(self, t, i, cv)
																			if cv == 1 then
																				subg_to_save = t
																			end
																		end,
																		value = (subg_to_save == "x2" and 1) or (subg_to_save == "x3" and 2) or (subg_to_save == "x4" and 3) or 1,
																		"x2",
																		"x3",
																		"x4",
																	},
																},
																iup.fill {
																	size = "%1",
																},
																iup.hbox {
																	iup.fill { },
																	iup.stationbutton {
																		title = bstr(23, "Load"),
																		action = function(self)
																			local timing = ACAM.load_timing(name_to_save, time_to_save, subg_to_save)
																			config.timing = timing
																			diag:update_content()
																			HideDialog(iup.GetDialog(self))
																		end,
																	},
																	iup.stationbutton {
																		title = bstr(27, "Cancel"),
																		action = function(self)
																			HideDialog(iup.GetDialog(self))
																		end,
																	},
																},
																iup.fill {
																	size = "%3",
																},
															},
															iup.fill {
																size = "%1",
															},
														},
													},
													iup.fill { },
												},
												iup.fill { },
											},
										}
										subdiag:map()
										ShowDialog(subdiag)
									end,
								},
							},
							iup.fill {
								size = "%3",
							},
						},
						iup.fill {
							size = "%1",
						},
					},
				},
				iup.fill { },
			},
			iup.fill { },
		}
	}
	
	diag:map()
	ShowDialog(diag)
end

RegisterUserCommand("ACAM", ACAM.open)








if isdeclared("lib") and (type(lib) == "table") and (lib[0] == "LME") then
	update_class()
	
	lib.require({{name = "babel", version = "0"}}, babel_func)
end